/*
*   NoCtlCpy: copy a string, converting control characters to
*   symbolic notation (^X),
*   printable characters (as in the ^X notation, but without the ^),
*   a fixed character,
*   or nothing (removing them entirely).
*   The high bit is always stripped before any other processing.
*   
*   Global variables: None.
*   
*   Input variables: 
*   src is the string to copy.
*   
*   max is the maximum number of characters to copy,
*   no including the terminating null character.
*   If max is -1, all characters are copied.
*   A terminating null character  is always added at the end.
*   
*   conversion specifies the conversion to apply
*   If conversion is 0, control characters are removed.
*   If conversion is -1, control characters are converted to capital letters.
*   If conversion is -2, control characters are converted to symbolic chars.
*   If conversion is any other value, control characters are converted to it.
*   
*   Output variables:
*   dest is the buffer to copy the string into.
*   
*   Return value:
*   dest is returned.
*   src and dest can be identical unless symbolic conversion is selected.
*/
char *NoCtlCpy(dest,src,max,conversion)
char *dest,*src;
unsigned max;
char conversion;
{
    char C;
    char *DestP = dest;

    for (; (C = *src & 0x7F) && max--; src++)
	if (isprint(C))
	    *(DestP++) = C;
	else switch (conversion) {
	case 0:
	    break;
	case (char) -2:
	    *(DestP++) = '^';
	case (char) -1:
	    *(DestP++) = (C == 0x7F ? '?' : C + '@');
	    break;
	default:
	    *(DestP++) = conversion;
	}
    *DestP = '\0';
    return(dest);
}
